@@ -15,6 +15,8 @@ import os
 import sys
 import time
 import warnings
+from collections.abc import Generator
+from contextlib import contextmanager
 from dataclasses import dataclass
 from datetime import date, timedelta
 from functools import lru_cache
@@ -98,6 +100,17 @@ Observation: "TypeAlias" = Union[InfoObservation, TestCaseObservation]
 TESTCASE_CALLBACKS: list[Callable[[Observation], None]] = []
 
 
+@contextmanager
+def with_observation_callback(
+    callback: Callable[[Observation], None],
+) -> Generator[None, None, None]:
+    TESTCASE_CALLBACKS.append(callback)
+    try:
+        yield
+    finally:
+        TESTCASE_CALLBACKS.remove(callback)
+
+
 def deliver_observation(observation: Observation) -> None:
     for callback in TESTCASE_CALLBACKS:
         callback(observation)
